# Third Party Stuff
from django.urls import path, include
from django.conf import settings
from rest_framework.routers import DefaultRouter

# phone_verify
from .api import VerificationViewSet

default_router = DefaultRouter(trailing_slash=False)
default_router.register('phone', VerificationViewSet, basename='phone')

urlpatterns = list(default_router.urls)



# -*- coding: utf-8 -*-

# Third Party Stuff
import pytest
from django_dynamic_fixture import G

from django.apps import apps


def create_verification(**kwargs):
    SMSVerification = apps.get_model('phone_verify', 'SMSVerification')
    verification = G(SMSVerification, **kwargs)
    return verification



# Generated by Django 2.1.7 on 2019-02-14 07:27

from django.db import migrations, models
import phonenumber_field.modelfields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SMSVerification',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
                ('otp', models.CharField(max_length=120, verbose_name='OTP')),
                ('phone_number', phonenumber_field.modelfields.PhoneNumberField(max_length=128, verbose_name='Phone Number')),
                ('session_code', models.CharField(max_length=500, verbose_name='Device Session Code')),
            ],
            options={
                'verbose_name': 'SMS Verification',
                'verbose_name_plural': 'SMS Verifications',
                'db_table': 'sms_verification',
                'ordering': ('-modified_at',),
            },
        ),
        migrations.AlterUniqueTogether(
            name='smsverification',
            unique_together={('otp', 'phone_number', 'session_code')},
        ),
    ]



# Standard Library
import uuid

# Third Party Stuff
from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.db import models
from django.utils.translation import ugettext_lazy as _
from phonenumber_field.modelfields import PhoneNumberField


class UUIDModel(models.Model):
    """ An abstract base class model that makes primary key `id` as UUID
    instead of default auto incremented number.
    """
    id = models.UUIDField(primary_key=True, editable=False, default=uuid.uuid4)

    class Meta:
        abstract = True


class TimeStampedUUIDModel(UUIDModel):
    """An abstract base class model that provides self-updating
    ``created`` and ``modified`` fields with UUID as primary_key field.
    """
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    modified_at = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        abstract = True


class SMSVerification(TimeStampedUUIDModel):
    otp = models.CharField(_('OTP'), max_length=120)
    phone_number = PhoneNumberField(_('Phone Number'))
    session_code = models.CharField(_('Device Session Code'), max_length=500)

    class Meta:
        db_table = 'sms_verification'
        verbose_name = _('SMS Verification')
        verbose_name_plural = _('SMS Verifications')
        ordering = ('-modified_at', )
        unique_together = ('otp', 'phone_number', 'session_code')

    def __str__(self):
        return '{}: {}'.format(str(self.phone_number), self.otp)



DJANGO_SETTINGS = {
    "SECRET_KEY": "change-me-later",
    "DATABASES": {
        "default": {
            "ENGINE": "django.db.backends.sqlite3",
        }
    },
    "ROOT_URLCONF": "phone_verify.urls",
    "INSTALLED_APPS": [
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "phone_verify",
    ],
    # PHONE VERIFICATION
    "PHONE_VERIFICATION": {
        'BACKEND': 'phone_verify.backends.twilio.TwilioBackend',
        'TWILIO_SANDBOX_TOKEN':'123456',
        'OPTIONS': {
            'SID': 'fake',
            'SECRET': 'fake',
            'FROM': '+14755292729'
        },
        'TOKEN_LENGTH': 6,
        'MESSAGE': 'Welcome to {app}! Please use security code {otp} to proceed.',
        'APP_NAME': 'Phone Verify',
        'OTP_EXPIRATION_TIME': 1  # In seconds only
    }
}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# Third party stuff
import json
import time
import pytest
from django.apps import apps
from django.urls import reverse

from . import factories as f

pytestmark = pytest.mark.django_db

OTP = '123456'
PHONE_NUMBER = '+13478379634'
SESSION_CODE = 'phone-auth-session-code'


def test_phone_registration_sends_message(client, mocker):
    url = reverse('phone-register')
    phone_number = PHONE_NUMBER
    data = {
        'phone_number': phone_number
    }
    twilio_api = mocker.patch('phone_verify.services.PhoneVerificationService.send_verification')

    response = client.post(url, data)

    assert response.status_code == 200
    assert twilio_api.called
    assert "session_code" in response.data
    SMSVerification = apps.get_model('phone_verify', 'SMSVerification')
    assert SMSVerification.objects.get(
        session_code=response.data['session_code'],
        phone_number=phone_number
    )


def test_otp_session_code_verification_api(client):
    verification = f.create_verification(
        otp=OTP, phone_number=PHONE_NUMBER,
        session_code=SESSION_CODE
    )
    url = reverse('phone-verify')
    data = {
        'phone_number': PHONE_NUMBER,
        'otp': OTP,
        'session_code': SESSION_CODE
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 200
    assert response.data['message'] == "OTP is valid."


def test_phone_verification_with_incomplete_payload(client):
    verification = f.create_verification(
        otp=OTP, phone_number=PHONE_NUMBER,
        session_code=SESSION_CODE
    )
    url = reverse('phone-verify')
    data = {
        'phone_number': PHONE_NUMBER
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 400
    response_data = json.loads(json.dumps(response.data))
    assert response_data['session_code'][0] == "This field is required."
    assert response_data['otp'][0] == "This field is required."

    data = {
        'otp': OTP
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 400
    response_data = json.loads(json.dumps(response.data))
    assert response_data['phone_number'][0] == "This field is required."


def test_phone_verification_with_incorrect_payload(client):
    verification = f.create_verification(
        otp=OTP, phone_number=PHONE_NUMBER,
        session_code=SESSION_CODE
    )
    url = reverse('phone-verify')
    # Payload with wrong session code
    data = {
        'phone_number': PHONE_NUMBER,
        'otp': OTP,
        'session_code': 'wrong-session-code'
    }
    response = client.json.post(url, data=data)
    response_data = json.loads(json.dumps(response.data))
    assert response.status_code == 400
    assert response_data['non_field_errors'][0] == "Session Code mis-match"

    # Payload with wrong OTP
    data = {
        'phone_number': PHONE_NUMBER,
        'otp': '999999',
        'session_code': SESSION_CODE
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 400
    response_data = json.loads(json.dumps(response.data))
    assert response_data['non_field_errors'][0] == "OTP is not valid"

    # Payload with incorrect phone_number
    data = {
        'phone_number': '+13478379632',
        'otp': OTP,
        'session_code': SESSION_CODE
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 400
    response_data = json.loads(json.dumps(response.data))
    assert response_data['non_field_errors'][0] == "OTP is not valid"


def test_otp_expired(client):
    verification = f.create_verification(
        otp=OTP, phone_number=PHONE_NUMBER,
        session_code=SESSION_CODE
    )
    time.sleep(2)
    url = reverse('phone-verify')
    data = {
        'phone_number': PHONE_NUMBER,
        'otp': OTP,
        'session_code': SESSION_CODE
    }
    response = client.json.post(url, data=data)
    assert response.status_code == 400
    response_data = json.loads(json.dumps(response.data))
    assert response_data['non_field_errors'][0] == "OTP has expired"



# -*- coding: utf-8 -*-

# Third Party Stuff
import pytest

# phone_verify Stuff
from phone_verify.services import PhoneVerificationService

pytestmark = pytest.mark.django_db


def test_message_generation_and_sending_service(client, mocker):
    service = PhoneVerificationService(phone_number='+13478379634')
    twilio_api = mocker.patch('phone_verify.backends.twilio.TwilioBackend.send_sms')
    service.send_verification('+13478379634', '123456')

    assert twilio_api.called



# -*- coding: utf-8 -*-

# Third Party Stuff
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.permissions import AllowAny

from .base import response
from .serializers import PhoneSerializer, SMSVerificationSerializer
from .services import send_otp_and_generate_session_code


class VerificationViewSet(viewsets.GenericViewSet):
    serializer_class = PhoneSerializer

    @action(detail=False, methods=['POST'], permission_classes=[AllowAny])
    def register(self, request):
        serializer = PhoneSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        session_code = send_otp_and_generate_session_code(str(serializer.validated_data['phone_number']))
        return response.Ok({'session_code': session_code})

    @action(detail=False, methods=['POST'], permission_classes=[AllowAny])
    def verify(self, request):
        serializer = SMSVerificationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        return response.Ok({'message': 'OTP is valid.'})



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

# Third Party Stuff
from django.contrib import admin

from .models import SMSVerification


@admin.register(SMSVerification)
class SMSVerificationAdmin(admin.ModelAdmin):
    list_display = ('id', 'otp', 'phone_number', 'created_at')
    search_fields = ('phone_number', )
    ordering = ('phone_number', )
    readonly_fields = ('otp', 'phone_number', 'session_code', 'created_at', 'modified_at')
